from http import HTTPStatus
from typing import TYPE_CHECKING

import pytest
from orjson import orjson
from starlette.requests import Request
from typing_extensions import Self

from api_helper.errors.app_error import ApplicationError
from api_helper.errors.app_error import handle_application_error

from ..fixure import utctime_mock  # noqa

if TYPE_CHECKING:
    from starlette.responses import Response


class TestApplicationError:
    def test_application_error(self: Self) -> None:
        exc: ApplicationError = ApplicationError()

        assert exc.status_code == HTTPStatus.INTERNAL_SERVER_ERROR
        assert exc.detail == "An unknown internal error has occurred while processing the request."

    def test_application_error_parameters(self: Self) -> None:
        exc: ApplicationError = ApplicationError(
            status_code=HTTPStatus.BAD_REQUEST, additional={"test": "test"}, list=[1, 2, 3]
        )

        assert exc.status_code == HTTPStatus.BAD_REQUEST
        assert exc.parameters == {"additional": {"test": "test"}, "list": [1, 2, 3]}

    def test_application_error_detail(self: Self) -> None:
        test_detail: str = "This is a test detail."
        exc: ApplicationError = ApplicationError(test_detail)

        assert exc.detail == test_detail

    @pytest.mark.usefixtures("utctime_mock")
    def test_application_handler(self: Self) -> None:
        response: Response = handle_application_error(Request({"type": "http"}), ApplicationError())

        assert response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR
        assert orjson.loads(response.body) == {
            "datetime": "2021-01-01T00:00:00+00:00",
            "error": {
                "code": 500,
                "detail": "An unknown internal error has occurred while processing the request.",
                "parameters": {},
            },
        }



from http import HTTPStatus

import orjson
import pytest
from typing_extensions import Self

from api_helper.responses import ErrorResponse
from api_helper.responses import SuccessResponse
from api_helper.responses import error_response
from api_helper.responses import success_response

from ..fixure import utctime_mock  # noqa


@pytest.mark.usefixtures("utctime_mock")
class TestResponseClass:
    def test_data_response(self: Self) -> None:
        response = SuccessResponse(data={"key": "value"})

        assert response.status_code == HTTPStatus.OK
        assert orjson.loads(response.body) == {
            "data": {"key": "value"},
            "datetime": "2021-01-01T00:00:00+00:00",
        }

    def test_empty_response(self: Self) -> None:
        response = SuccessResponse()

        assert response.status_code == HTTPStatus.OK
        assert orjson.loads(response.body) == {
            "data": "",
            "datetime": "2021-01-01T00:00:00+00:00",
        }

    def test_response_code(self: Self) -> None:
        response = SuccessResponse(status_code=HTTPStatus.NO_CONTENT)

        assert response.status_code == HTTPStatus.NO_CONTENT
        assert orjson.loads(response.body) == {
            "data": "",
            "datetime": "2021-01-01T00:00:00+00:00",
        }

    def test_error_response_key(self: Self) -> None:
        response = ErrorResponse()

        assert response.status_code == HTTPStatus.BAD_REQUEST
        assert orjson.loads(response.body) == {
            "error": "",
            "datetime": "2021-01-01T00:00:00+00:00",
        }


@pytest.mark.usefixtures("utctime_mock")
class TestResponseFunction:
    def test_success_response(self: Self) -> None:
        response = success_response()

        assert response.status_code == HTTPStatus.OK
        assert orjson.loads(response.body) == {
            "data": "success",
            "datetime": "2021-01-01T00:00:00+00:00",
        }

    def test_success_response_list(self: Self) -> None:
        response = success_response([])

        assert response.status_code == HTTPStatus.OK
        assert orjson.loads(response.body) == {
            "data": [],
            "datetime": "2021-01-01T00:00:00+00:00",
        }

    def test_success_response_dict(self: Self) -> None:
        response = success_response({})

        assert response.status_code == HTTPStatus.OK
        assert orjson.loads(response.body) == {
            "data": {},
            "datetime": "2021-01-01T00:00:00+00:00",
        }

    def test_success_error_with_error(self: Self) -> None:
        with pytest.raises(ValueError) as error:
            success_response(status_code=HTTPStatus.BAD_REQUEST)

        assert str(error.value) == "4xx and 5xx are not a success response."

    def test_error_response(self: Self) -> None:
        response = error_response()

        assert response.status_code == HTTPStatus.BAD_REQUEST
        assert orjson.loads(response.body) == {
            "error": "",
            "datetime": "2021-01-01T00:00:00+00:00",
        }

    def test_error_response_with_success(self: Self) -> None:
        with pytest.raises(ValueError) as error:
            error_response(status_code=HTTPStatus.TEMPORARY_REDIRECT)

        assert str(error.value) == "Only 4xx and 5xx are available for error response."
